precision highp float;

varying vec2 uTexCoord;

uniform sampler2D uTexture;

uniform float uDistortion;  // [0.0, 1.0]
uniform float uDirection;
uniform float uNScale;
uniform float uNSpeed;
uniform float uAspectRatio;
uniform int uComplexity;

uniform float uTime;
uniform vec2 uResolution;

#define PI 3.1415926535898


vec2 random(vec2 st) {
    st = vec2(dot(st, vec2(72.1,111.7)), dot(st, vec2(69.5,83.3)));
    return -1.0 + 2.0 * fract(sin(st) * 43452.576843);
}

vec2 gradientNoise(vec2 st) {
    vec2 i = floor(st);
    vec2 f = fract(st);

    vec2 u = f * f * (3.0 - 2.0 * f);
    return mix( mix( random(i + vec2(0.0,0.0)), random(i + vec2(1.0,0.0)), u.x ),
                mix( random(i + vec2(0.0,1.0)), random(i + vec2(1.0,1.0)), u.x ), u.y );
}

vec2 fbm(vec2 st, vec2 fbmOff) {
    vec2 value = vec2(0.0);
    st.x *= uAspectRatio * uResolution.x / uResolution.y;
    st *= uNScale;

    float amplitude = 0.5;
    float frequency = 3.0;
    float gain = 0.424;

    mat2 m = mat2(0.8, 0.6, -0.6, 0.8);
    for (int i = 0; i < uComplexity; i++) {
        vec2 temp = st + fbmOff;
        vec2 r = sin(gradientNoise(temp) * PI * 0.5);
        r = r * abs(r);
        value += amplitude * r;
        st = m * st * frequency;
        fbmOff *= frequency;
        amplitude *= gain;
    }
    return value;
}

void main() {
    vec2 st = uTexCoord;
    float time = mod(uTime, 10000.0);

    float dir = uDirection * PI / 180.0;
    vec2 rot = vec2(-cos(dir), -sin(dir));
    vec2 offset = rot * time * uNSpeed;
    vec2 pos = st + fbm(st + offset * 0.001, offset * vec2(0.0108, 0.012)) * uDistortion;

    gl_FragColor = texture2D(uTexture, pos);
}
