precision highp float;

varying vec2 uTexCoord;

uniform sampler2D uTexture;

uniform float uDistortion;  // [0.0, 1.0]
uniform float uDirection;
uniform float uNScale;
uniform float uNSpeed;
uniform float uAspectRatio;
uniform int uComplexity;

uniform float uTime;
uniform vec2 uResolution;

#define PI 3.1415926535898


vec2 random(vec2 st) {
    st = vec2(dot(st, vec2(12.9898,12.1414)), dot(st, vec2(29.5, 83.3)));
    return -1.0 + 2.0 * fract(sin(st) * 43758.5453123);
}

vec2 gradientNoise(vec2 st) {
    vec2 i = floor(st);
    vec2 f = fract(st);

    vec2 u = f * f * (3.0 - 2.0 * f);

    return mix( mix( random(i + vec2(0.0,0.0)), random(i + vec2(1.0,0.0)), u.x),
                mix( random(i + vec2(0.0,1.0)), random(i + vec2(1.0,1.0)), u.x), u.y);
}

vec2 fbm(vec2 st, vec2 fbmOff) {
    vec2 value = vec2(0.0);
    vec2 offset = gradientNoise(st * 14.5 + fbmOff * 0.1) * 3.813;
    st.x *= uAspectRatio * uResolution.x / uResolution.y;
    st = st * uNScale * vec2(1.6749, 2.1314) + vec2(-12.1314, 25.4854);

    float amplitude = 0.45;
    float frequency = 4.0;
    float gain = 0.424;

    float a75 = 75.0 * PI / 180.0;
    mat2 rot = mat2(cos(a75), sin(a75),
                    -sin(a75), cos(a75));

    for (int i = 0; i < uComplexity; i++) {
        vec2 temp = st + fbmOff;
        value += amplitude * gradientNoise(temp);
        st = rot * st * frequency + offset;
        fbmOff *= frequency;
        amplitude *= gain;
    }
    return value;
}

void main() {
    vec2 st = uTexCoord;
    float time = mod(uTime, 10000.0);

    float dir = uDirection * PI / 180.0;
    vec2 rot = vec2(-cos(dir), -sin(dir));
    vec2 offset = rot * time * uNSpeed;
    vec2 pos = st + fbm(st + offset * 0.001, offset * vec2(0.0168, 0.0180)) * uDistortion;

    gl_FragColor = texture2D(uTexture, pos);
}
