precision highp float;

varying float uProgress;
varying float uDistance;

uniform vec2 uResolution;

uniform float uStartWidth;
uniform float uEndWidth;

uniform float uStartAlpha;
uniform float uEndAlpha;

uniform float uThickness;
uniform float uDiffusion;

uniform vec4 uDiffuseColor;
uniform vec4 uShineColor;

float funcS(float p) {
    return smoothstep(0.0, 2.0, p) * 0.833 + 0.975;
}

float funcE(float p) {
    return 1.0 - 0.667 / exp(1.0) + 0.667 / exp(p * 0.5);
}

void main() {
    float width = uStartWidth - uProgress * (uStartWidth - uEndWidth);
    if (uDistance > width) {
        discard;
    }
    float alpha = uStartAlpha - uProgress * (uStartAlpha - uEndAlpha);

    float thickness = uThickness * min(uResolution.x, uResolution.y) * 0.5;
    float dis = uDistance / width * thickness;

    float ratio = thickness * 0.15;
    float off = min(5.0, ratio);
    float dif = off * sqrt(uDiffusion);
    float undif = ratio - dif;

    float greater = step(undif, dis);
    float dud = dis - undif;
    dis = min(dis, undif) + greater * dud * 0.045;

    float difFle = dif * funcS(uDiffusion);
    float undifFle = ratio - difFle;
    float f = 1.0 - clamp((dis - undifFle) / (difFle + step(difFle, 0.0)), 0.0, 1.0);

    float difFle2 = difFle * funcE(uDiffusion);
    float undifFle2 = ratio - difFle2;
    float f2 = 1.0 - clamp((dis - undifFle2) / (difFle2 + step(difFle2, 0.0)), 0.0, 1.0);
    float f3 = smoothstep(0.6, 1.0 + sqrt(uDiffusion * 0.5) * 0.075, f2);
    float constract = max(0.0, undifFle2);
    float f4 = clamp(0.0, constract, dis);
    f4 = f4 / (constract + step(constract, 0.0));

    float minP = min(min(uDiffuseColor.r, uDiffuseColor.g), uDiffuseColor.a);
    vec3 mediumColor = mix(uDiffuseColor.rgb, uShineColor.rgb, 1.0 - step(uDiffuseColor.rgb, vec3(minP)));
    mediumColor = uDiffuseColor.rgb + (mediumColor - uDiffuseColor.rgb) * 0.87;

    vec3 mixRgb = vec3(0.0);
    if (f4 < 1.0) {
        mixRgb = mix(uShineColor.rgb, mediumColor, pow(f4, 1.25));
    } else {
        mixRgb = mix(uDiffuseColor.rgb, mediumColor, f3);
    }
    vec4 color = vec4(mixRgb, mix(0.0, uShineColor.a, pow(f, 3.0)));
    color.a *= step(dis, ratio) * (1.0 - step(thickness, 0.0));

    color.a *= alpha;
    gl_FragColor = vec4(color.rgb * color.a, color.a);
}
