#extension GL_OES_standard_derivatives : enable

precision highp float;

#define PI 3.141592653

const int u_Platform = 1;
const vec4 white = vec4(1.0, 1.0, 1.0, 1.0);
const vec4 black = vec4(0.0, 0.0, 0.0, 1.0);
const vec4 transparent = vec4(0.0);

varying vec2 textureCoordinate;
varying vec2 textureCoordinate2;

// 画布纹理
uniform sampler2D inputImageTexture;
// layer纹理
uniform sampler2D inputImageTexture2;

// 是否fade
uniform float fade;
// 动画进度(0-1)
uniform float progress;
// layer框大小(像素尺度如400,600)
uniform vec2 texSize;
// layer框坐标(以0.5,0.5为锚点,如200,300)
uniform vec2 texPos;
//uniform vec2 anchorPos;
// layer框旋转角度(角度制)
uniform float texRotate;
// layer框运动方向(不要求归一化)
uniform vec2 direction;
// 平移模糊程度(如当前速度与匀速的比值)
uniform float moveAcc;
// 旋转模糊程度(当前旋转速度与匀速的比值)
uniform float angleAcc;
// 缩放模糊程度(当前缩放速度与匀速的比值)
uniform float scaleAcc;
// 0代表全透明/1代表完全不透明
uniform float opacity;
// 画布尺寸(如1242.0,2208.0)
uniform vec2 canvasSize;
//// 纹理取样坐标组(如 0.0,0.0,1.0,0.0,0.0,1.0,1.1,1.1)
//uniform float textureParams[8];

// 可能layer和canvas是一样大的，但layer里除了局部区域是运动模糊的作用目标，其他地方是透明的， 不需要进行计算
uniform vec2 realBlurTargetPos;
uniform vec2 realBlurTargetSize;

///****调试用的不用管******/
//uniform vec2 u_Size;
//uniform float u_Time;
//uniform float u_TotalTime;
/**********************/

vec2 adjust(vec2 uv) {
    return u_Platform == 0 ? vec2(uv.x, 1.0-uv.y): uv;
}

vec2 position(vec2 uv, vec4 slot, vec4 parent) {
    return (uv*parent.zw-vec2(slot.x, parent.w-slot.y-slot.w))/slot.zw;
}

vec2 position(vec2 uv, vec4 slot, vec4 parent, vec2 anchor) {
    return position(uv, vec4(slot.xy-anchor, slot.zw), parent);
}

vec2 rotate(vec2 src, float angle, vec2 ratio, vec2 anchor) {
    src -= vec2(anchor.x, 1.0-anchor.y);
    return (src*ratio*mat2(cos(angle), -sin(angle), sin(angle), cos(angle)))*(1.0/ratio)+vec2(anchor.x, 1.0-anchor.y);
}

vec2 rotate(vec2 src, float angle, vec2 ratio) {
    return rotate(src, angle, ratio, vec2(0.5));
}

vec2 rotate(vec2 src, float angle) {
    return rotate(src, angle, vec2(1.0, 1.0));
}

bool inside(vec2 pos) {
    return pos.x >= 0.0 && pos.x <= 1.0 && pos.y >= 0.0 && pos.y <= 1.0;
}

void blend(in vec4 src, inout vec4 res, float opacity) {
    res = vec4(src.rgb*opacity*(1.-step(src.a, 0.))+ (1.-src.a*opacity)*res.rgb, max(res.a, src.a*opacity));
}
void blend(in vec4 src, inout vec4 res) {
    blend(src, res, 1.);
}

float linear(float t, float b, float c, float d) {
    return mix(b, c, clamp(t/d, 0.0, 1.0));
}

float rand(vec2 xy) {
    return fract(sin(dot(xy, vec2(12.9898, 78.233)))*43758.5453123);
}

vec2 scale(vec2 uv, vec2 anchor, vec2 scale) {
    return (uv-anchor)/scale  + anchor;
}

vec2 scale(vec2 uv, vec2 anchor, float scale) {
    return (uv-anchor)/scale  + anchor;
}

vec2 transform(vec2 uv, float params[5], vec2 ratio) {
    vec2 pos = uv;
    pos.xy -= (vec2(params[2], .5) - vec2(.5, params[3]));
    pos = scale(pos, vec2(.5), vec2(params[0], params[1]));
    pos = rotate(pos, radians(params[4]), ratio*vec2(params[0], params[1]));
    return pos;
}

vec4 texture2D(sampler2D tex, vec2 uv, float params[5], vec2 ratio) {
    vec2 pos = adjust(uv);
    pos = transform(pos, params, ratio);
    float pct = step(0., pos.x)*step(0., pos.y)*step(0., 1.-pos.x)*step(0., 1.-pos.y);
    return mix(vec4(0.), texture2D(tex, adjust(pos)), pct);
}

vec4 texture2D(sampler2D tex, vec2 uv, float params[8]) {
    vec2 pos = adjust(uv);
    //    pos = transform(pos, params, ratio);

    vec2 p0 = vec2(params[0], params[1]);
    vec2 p1 = vec2(params[2], params[3]);
    vec2 p2 = vec2(params[4], params[5]);
    vec2 p3 = vec2(params[6], params[7]);

    p0.y = 1. - p0.y;
    p1.y = 1. - p1.y;
    p2.y = 1. - p2.y;
    p3.y = 1. - p3.y;

    float x = pos.x;
    float y = 1.0-pos.y;

    pos.x = mix(mix(p0.x, p1.x, x), mix(p2.x, p3.x, x), y);
    pos.y = mix(mix(p0.y, p2.y, y), mix(p1.y, p3.y, y), x);

    float pct = step(0., pos.x)*step(0., pos.y)*step(0., 1.-pos.x)*step(0., 1.-pos.y);
    return mix(vec4(0.), texture2D(tex, adjust(pos)), pct);
}

float easeOutSine(float t, float b, float c, float d) {
    float x = clamp(t/d, 0., 1.);
    return mix(b, c, sin(x * PI / 2.));
}

float easeInOutSine(float t, float b, float c, float d) {
    float x = clamp(t/d, 0., 1.);
    return mix(b, c, -(cos(PI * x) - 1.) / 2.);
}

void main(){
//    float startFrame = 0.;
//    vec2 StageSize = vec2(1242.0, 2208.0);
//    float time = mod(u_Time, u_TotalTime);
//    float frame = time*24.;
//    float duration = 16.;
//    float lastFrame = max(0., frame - 1.);
//
//    // 纹理大小
//    float texW = 1000.;
//    float texH = 1000.;
//    // 动画帧率
//    //    float duration = 16.;
//
//    // 是否fade
//    float fade0 = 1.;
//    // 动画进度(0-1)
//    float progress0 = min(1., frame / duration);
//
//    vec2 from = vec2(-1000., 300.);
//    vec2 to = vec2(StageSize.x-300., 900.);
//
//    vec2 direction0 = to - from;
//    // 当前方向
//    vec2 normDirection = direction0/length(direction0);
//
//    // 当前坐标
//    vec2 texPos0 = vec2(easeInOutSine(frame-startFrame, from.x, to.x, duration), easeInOutSine(frame-startFrame, from.y, to.y, duration));
//
//    float now = easeInOutSine(frame-startFrame, from.x, to.x, duration);
//    float last = easeInOutSine(lastFrame-startFrame, from.x, to.x, duration);
//    float speed = abs(last-now);
//    float linear_speed =  abs(from.x-to.x) / duration;
//
//    // 当前速度与匀速的比值(max(speed, linear_speed) / linear_speed)
////    float moveAcc0 = max(speed, linear_speed) / linear_speed;
//    //            moveAcc = 1.;
//
////    float fromAngle = 0.;
////    float toAngle = 360.;
//    float nowAngle = easeInOutSine(frame-startFrame, fromAngle, toAngle, duration);
//    // 当前旋转速度与匀速的比值
////    float angleAcc0 = max(speed, linear_speed) / linear_speed;
//    //            angleAcc = 1.;
//
//    float totexW = 1000.;
//    float totexH = 1000.;
//    // 当前纹理大小
//    vec2 nowTexSize = vec2(easeInOutSine(frame-startFrame, texW, totexW, duration), easeInOutSine(frame-startFrame, texH, totexH, duration));
//    // 当前缩放速度与匀速的比值
////    float scaleAcc0 = max(speed, linear_speed) / linear_speed;
//    //            scaleAcc = 1.;
//
//    // 是否fade
//    float fade = 0.;//fade0;
////    // 动画进度(0-1)
////    float progress = progress0;
//    // layer框大小
//    vec2 texSize = nowTexSize;
//    // layer框坐标(以0.5，0.5为锚点)
//    vec2 texPos = texPos0;
//    // layer框旋转角度
//    float texRotate = nowAngle;
////    // layer框运动方向
////    vec2 direction = normDirection;
//    // 平移模糊程度(如当前速度与匀速的比值)
//    float moveAcc = 1.;
////    // 旋转模糊程度(当前旋转速度与匀速的比值)
//    float angleAcc = 1.3;
////    // 缩放模糊程度(当前缩放速度与匀速的比值)
//    float scaleAcc = 1.;
//    // 0代表全透明/1代表完全不透明
//    float opacity = 1.0;
////    // 画布尺寸
////    vec2 canvasSize = StageSize;
    // 纹理取样坐标组
//    float textureParams[8];
//    textureParams[0] = 0.;
//    textureParams[1] = 1.;
//    textureParams[2] = 0.;
//    textureParams[3] = 0.;
//    textureParams[4] = 1.;
//    textureParams[5] = 1.;
//    textureParams[6] = 1.;
//    textureParams[7] = 0.;

    float fade = 0.0;



    vec2 uv = textureCoordinate;
    vec4 root = vec4(vec2(0.), canvasSize);

    //    vec4 color = white;
    vec4 color = texture2D(inputImageTexture, adjust(uv));

    //    direction = vec2(0.0);
    //    float directionLength = length(direction);
    //    vec2 normDirect = mix(direction/directionLength, vec2(0.), step(directionLength, 0.));
    vec2 normDirect = normalize(direction);

    vec2 anchorPos = texPos / canvasSize;

    vec4 pruneSlot = vec4(realBlurTargetPos.x, realBlurTargetPos.y, realBlurTargetSize.x*1.5,
                    realBlurTargetSize.y*1.5);
//    vec2 prunePos = position(uv, pruneSlot, root, pruneSlot.zw*vec2(.5, .5));
//    prunePos = rotate(prunePos, radians(texRotate), pruneSlot.zw);
    vec2 prunePos = position(uv, pruneSlot, root, pruneSlot.zw*vec2(.5, .5));
    prunePos = rotate(prunePos, radians(texRotate), pruneSlot.zw);
    if (inside(prunePos)) {
//        color = vec4(0., 1., 0., 1.);
        int itercount = 12;
        vec4 layer = vec4(0.);
        float rnd = rand(uv);
        float total = 0.0;
        for (int i=0;i<=itercount;i++){
            float percent = (float(i)+rnd)/float(itercount);
            //                float weight = 4.0 * (percent - percent * percent);
            float weight = 2.0 * (percent - percent * percent) + 0.5;
            vec2 blurPos = texPos - normDirect*texSize*0.3* (moveAcc - 1.) * (percent - 0.5);
            vec2 size = texSize - texSize*0.3 * (scaleAcc - 1.)* (percent - 0.5);
            vec4 slot = vec4(blurPos.x, blurPos.y, size.x, size.y);
//            vec2 pos = position(uv, slot, root, slot.zw*vec2(.5, .5));
            vec2 pos = position(uv, slot, root, slot.zw*anchorPos);
            float angle = 0.0 - 50. * (angleAcc - 1.) * (percent - 0.5);
            pos = rotate(pos, radians(angle), slot.zw, anchorPos);
//            if (distance(uv, vec2(anchorPos.x , 1.- anchorPos.y)) < 0.01) {
//                gl_FragColor = vec4(1.,0.,0.,1.);
//                return;
//            }
            //            pos = rotate(pos, radians(angle), slot.zw);
            if (inside(pos)) {
                layer += texture2D(inputImageTexture2, adjust(pos))*weight;
//                gl_FragColor = vec4(1.);
//                return;
            } else {
                layer += color*weight;
            }
            total += weight;
        }
        layer /= total;
        blend(layer, color, mix(progress, opacity, step(fade, 0.5)));

    }

    gl_FragColor = color;

}