precision highp float;

varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform highp vec2 referPoint;
uniform highp vec2 point1;
uniform highp vec2 point2;
uniform highp float radius;
uniform highp float ratio;
uniform bool eraseMode;

highp float PointToSegDist(vec2 p, mediump vec2 p1, mediump vec2 p2) {
    p.y = p.y / ratio;
    p1.y = p1.y / ratio;
    p2.y = p2.y / ratio;
    float cross = (p2.x - p1.x) * (p.x - p1.x) + (p2.y - p1.y) * (p.y - p1.y);
    if (cross <= 0.0) return distance(p, p1);

    float d2 = (p2.x - p1.x) * (p2.x - p1.x) + (p2.y - p1.y) * (p2.y - p1.y);
    if (cross >= d2) return distance(p, p2);

    float r = cross / d2;
    vec2 pp = vec2(p1.x + (p2.x - p1.x) * r, p1.y + (p2.y - p1.y) * r);
    return distance(p, pp);
}

void main() {
    float calcRadius = PointToSegDist(textureCoordinate, point1, point2);
    if (eraseMode) {
        vec4 texture1Color = texture2D(inputImageTexture2, textureCoordinate);
        float alphaMulti = smoothstep(0.5 * radius, radius, calcRadius);
        vec4 texture1ColorCmp = texture2D(inputImageTexture2, point2);
        float colorDistance = distance(texture1Color.rgb, texture1ColorCmp.rgb);
        float alphaMulti2 = smoothstep(0.10, 0.16, colorDistance);
        float alphaMulti3 = pow(alphaMulti, 2.0) + pow(alphaMulti2, 2.0);
        alphaMulti3 = clamp(alphaMulti3, 0.0, 1.0) - (1.0 - smoothstep(0.1 * radius, 0.3 * radius, calcRadius));
        alphaMulti3 = clamp(alphaMulti3, 0.0, 1.0);
        vec4 texture2Color = texture2D(inputImageTexture, textureCoordinate);
        gl_FragColor = texture2Color * alphaMulti3;
        return;
    }
    vec4 texture1Color = texture2D(inputImageTexture, textureCoordinate);
    if (calcRadius <= radius) {
        vec4 originColor = texture2D(inputImageTexture2, textureCoordinate);
        float alphaMulti2 = smoothstep(radius * 0.9, radius, calcRadius);
        float maxalpha = max(texture1Color.a, originColor.a * (1.0 - alphaMulti2));
        gl_FragColor = vec4(originColor.rgb, maxalpha);
        return;
    }
    gl_FragColor = texture1Color;
}
